/*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

 Module:       FGTurboProp.cpp
 Author:       Jiri "Javky" Javurek
               based on SimTurbine and Turbine engine from David Culp
 Date started: 05/14/2004
 Purpose:      This module models a turbo propeller engine.

 ------------- Copyright (C) 2004  (javky@email.cz) ---------

 This program is free software; you can redistribute it and/or modify it under
 the terms of the GNU General Public License as published by the Free Software
 Foundation; either version 2 of the License, or (at your option) any later
 version.

 This program is distributed in the hope that it will be useful, but WITHOUT
 ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
 details.

 You should have received a copy of the GNU General Public License along with
 this program; if not, write to the Free Software Foundation, Inc., 59 Temple
 Place - Suite 330, Boston, MA  02111-1307, USA.

 Further information about the GNU General Public License can also be found on
 the world wide web at http://www.gnu.org.

FUNCTIONAL DESCRIPTION
--------------------------------------------------------------------------------

This class descends from the FGEngine class and models a Turbo propeller engine
based on parameters given in the engine config file for this class

HISTORY
--------------------------------------------------------------------------------
05/14/2004  Created

//JVK (mark)

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
INCLUDES
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/

#include <vector>
#include <sstream>
#include "FGTurboProp.h"

#include "FGPropeller.h"

#include "../Main/fg_props.hxx"
namespace JSBSim {

static const char *IdSrc = "$Id: FGTurboProp.cpp,v 1.0 2005/09/25 20:00:00 javky Exp $";
static const char *IdHdr = ID_TURBOPROP;

/*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
CLASS IMPLEMENTATION
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/


FGTurboProp::FGTurboProp(FGFDMExec* exec, FGConfigFile* cfg, int engine_number)
	: FGEngine(exec,engine_number)
{
typedef double (FGTurboProp::*PMF)(int) const;  
	
	SetDefaults();

  Load(cfg);
  Debug(0);
EngineNumber = engine_number	;
}

//%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

FGTurboProp::~FGTurboProp()
{
  Debug(1);
}

//%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
// The main purpose of Calculate() is to determine what phase the engine should
// be in, then call the corresponding function. 

double FGTurboProp::Calculate(void)
{
	TAT = (Auxiliary->GetTotalTemperature() - 491.69) * 0.5555556;
	dt = State->Getdt() * Propulsion->GetRate();
	
	ThrottleCmd = FCS->GetThrottleCmd(EngineNumber);
		
	Prop_RPM = Thruster->GetRPM() * Thruster->GetGearRatio();
	if (Thruster->GetType() == FGThruster::ttPropeller) {
		((FGPropeller*)Thruster)->SetAdvance(FCS->GetPropAdvance(EngineNumber));
		((FGPropeller*)Thruster)->SetFeather(FCS->GetPropFeather(EngineNumber));
		((FGPropeller*)Thruster)->SetReverse(Reversed);
		if (Reversed) {
			((FGPropeller*)Thruster)->SetReverseCoef(ThrottleCmd);
		} else {
			((FGPropeller*)Thruster)->SetReverseCoef(0.0);
		}
	}
  
	if (Reversed) {
		if (ThrottleCmd < BetaRangeThrottleEnd) {
				ThrottleCmd = 0.0;  // idle when in Beta-range
		} else {
			// when reversed:
			ThrottleCmd = (ThrottleCmd-BetaRangeThrottleEnd)/(1-BetaRangeThrottleEnd) * ReverseMaxPower;
		}
	}

	// When trimming is finished check if user wants engine OFF or RUNNING
	if ((phase == tpTrim) && (dt > 0)) {
		if (Running && !Starved) {
			phase = tpRun;
			N2 = IdleN2;
			N1 = IdleN1;
			OilTemp_degK = 366.0;  
			Cutoff = false;
		} else {
			phase = tpOff;
			Cutoff = true;
			Eng_ITT_degC = TAT;
			Eng_Temperature = TAT;
			OilTemp_degK = TAT+273.15;
		}
	}
  
	if (!Running && Starter) {
		if (phase == tpOff) { 
			phase = tpSpinUp;
			if (StartTime < 0) StartTime=0;
		}
	}
	if (!Running && !Cutoff && (N1 > 15.0)) {
		phase = tpStart;
		StartTime = -1;
	}
	if (Cutoff && (phase != tpSpinUp)) phase = tpOff;
	if (dt == 0) phase = tpTrim;
	if (Starved) phase = tpOff;
	if (Condition >= 10) {
		phase = tpOff;StartTime=-1;
	}
  
	if (Condition < 1) {
		if (Ielu_max_torque > 0
			&& -Ielu_max_torque > ((FGPropeller*)(Thruster))->GetTorque()
			&& ThrottleCmd >= OldThrottle ) {
			ThrottleCmd = OldThrottle - 0.1 * dt; //IELU down
			Ielu_intervent = true;
		} else if (Ielu_max_torque > 0 && Ielu_intervent && ThrottleCmd >= OldThrottle) {
			ThrottleCmd = OldThrottle;
			ThrottleCmd = OldThrottle + 0.05 * dt; //IELU up
			Ielu_intervent = true;
		} else {
			Ielu_intervent = false;
		}
	} else {
		Ielu_intervent = false;
	}
	OldThrottle = ThrottleCmd;
	
	switch (phase) {
		case tpOff:    Eng_HP = Off(); break;
		case tpRun:    Eng_HP = Run(); break;
		case tpSpinUp: Eng_HP = SpinUp(); break;
		case tpStart:  Eng_HP = Start(); break;
		default: Eng_HP = 0;
	}   

	//printf ("EngHP: %lf / Requi: %lf\n",Eng_HP,Prop_Required_Power);
	return Thruster->Calculate((Eng_HP * hptoftlbssec)-Thruster->GetPowerRequired());
}

//%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

double FGTurboProp::Off(void)
{
	double qbar = Auxiliary->Getqbar();
	Running = false; EngStarting = false;

	FuelFlow_pph = Seek(&FuelFlow_pph, 0, 800.0, 800.0);

	//allow the air turn with generator
	N1 = ExpSeek(&N1, qbar/15.0, Idle_Max_Delay*2.5, Idle_Max_Delay * 5);

	OilTemp_degK = ExpSeek(&OilTemp_degK,273.15 + TAT, 400 , 400);

	Eng_Temperature = ExpSeek(&Eng_Temperature,TAT,300,400);
	double ITT_goal = ITT_N1->GetValue(N1,0.1) + ((N1>20) ? 0.0 : (20-N1)/20.0 * Eng_Temperature);
	Eng_ITT_degC  = ExpSeek(&Eng_ITT_degC,ITT_goal,ITT_Delay,ITT_Delay*1.2);
	
	OilPressure_psi = (N1/100.0*0.25+(0.1-(OilTemp_degK-273.15)*0.1/80.0)*N1/100.0) / 7692.0e-6; //from MPa to psi
	Augmentation = false;

	ConsumeFuel(); //for possible setting Starved = false when fueltank
				   //is refilled (fuel crossfeed etc.)
	
	if (Prop_RPM>5) return -0.012; // friction in engine when propeller spining (estimate)
	return 0.0; 
}

//%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

double FGTurboProp::Run(void)
{
  double idlethrust, milthrust, thrust, EngPower_HP, eff_coef;
  Running = true; Starter = false; EngStarting = false;
 
	
//---
	double old_N1 = N1;
	N1 = ExpSeek(&N1, IdleN1 + ThrottleCmd * N1_factor, Idle_Max_Delay, Idle_Max_Delay * 2.4);
	
	EngPower_HP = EngPower_RPM_N1->GetValue(Prop_RPM,N1);
	EngPower_HP *= ThrustTables[ENG_POWER_COEF_AIRPRESSURE_AIRSPEED]->TotalValue();
	if (EngPower_HP > MaxPower) EngPower_HP = MaxPower;

	eff_coef = 9.333 - (N1)/12; // 430%Fuel at 60%N1
	FuelFlow_pph = FuelConsumption * EngPower_HP * eff_coef;
	
	Eng_Temperature = ExpSeek(&Eng_Temperature,Eng_ITT_degC,300,400);
	double ITT_goal = ITT_N1->GetValue((N1-old_N1)*300+N1,1);
	Eng_ITT_degC  = ExpSeek(&Eng_ITT_degC,ITT_goal,ITT_Delay,ITT_Delay*1.2);
	
	OilPressure_psi = (N1/100.0*0.25+(0.1-(OilTemp_degK-273.15)*0.1/80.0)*N1/100.0) / 7692.0e-6; //from MPa to psi
//---	
  EPR = 1.0 + thrust/MilThrust;

		OilTemp_degK = Seek(&OilTemp_degK, 353.15, 0.4-N1*0.001, 0.04);
	
  ConsumeFuel();
  if (Cutoff) phase = tpOff;
  if (Starved) phase = tpOff;

  return EngPower_HP;
}
        
//%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

double FGTurboProp::SpinUp(void)
{
	double EngPower_HP;
	Running = false; EngStarting = true;
	FuelFlow_pph = 0.0;
	
	if (!GeneratorPower) {
		EngStarting=false;
		phase=tpOff;
		StartTime = -1;
		return 0.0;
	}
	
	N1 = ExpSeek(&N1, StarterN1, Idle_Max_Delay * 6, Idle_Max_Delay * 2.4);

	Eng_Temperature = ExpSeek(&Eng_Temperature,TAT,300,400);
	double ITT_goal = ITT_N1->GetValue(N1,0.1) + ((N1>20) ? 0.0 : (20-N1)/20.0 * Eng_Temperature);
	Eng_ITT_degC  = ExpSeek(&Eng_ITT_degC,ITT_goal,ITT_Delay,ITT_Delay*1.2);

	OilTemp_degK = ExpSeek(&OilTemp_degK,273.15 + TAT, 400 , 400);
	
	OilPressure_psi = (N1/100.0*0.25+(0.1-(OilTemp_degK-273.15)*0.1/80.0)*N1/100.0) / 7692.0e-6; //from MPa to psi
	NozzlePosition = 1.0;

	EngPower_HP = EngPower_RPM_N1->GetValue(Prop_RPM,N1);
	EngPower_HP *= ThrustTables[ENG_POWER_COEF_AIRPRESSURE_AIRSPEED]->TotalValue();
	if (EngPower_HP > MaxPower) EngPower_HP = MaxPower;

	if (StartTime>=0) StartTime+=dt;
	if (StartTime > MaxStartingTime && MaxStartingTime > 0) { //start failed due timeout
		phase = tpOff;
		StartTime = -1;
	}

	ConsumeFuel(); //for possible setting Starved = false when fueltank
				   //is refilled (fuel crossfeed etc.)

	return EngPower_HP;
}

//%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

double FGTurboProp::Start(void)
{
	double EngPower_HP,eff_coef;
	EngStarting = false;
  if ((N1 > 15.0) && !Starved) {       // minimum 15% N2 needed for start
	double old_N1 = N1;
    Cranking = true;                   // provided for sound effects signal
    if (N1 < IdleN1) {
	EngPower_HP = EngPower_RPM_N1->GetValue(Prop_RPM,N1);
	EngPower_HP *= ThrustTables[ENG_POWER_COEF_AIRPRESSURE_AIRSPEED]->TotalValue();
	if (EngPower_HP > MaxPower) EngPower_HP = MaxPower;
      N1 = ExpSeek(&N1, IdleN1*1.1, Idle_Max_Delay*4, Idle_Max_Delay * 2.4);
	eff_coef = 9.333 - (N1)/12; // 430%Fuel at 60%N1
	FuelFlow_pph = FuelConsumption * EngPower_HP * eff_coef;
	Eng_Temperature = ExpSeek(&Eng_Temperature,Eng_ITT_degC,300,400);
	double ITT_goal = ITT_N1->GetValue((N1-old_N1)*300+N1,1);
	Eng_ITT_degC  = ExpSeek(&Eng_ITT_degC,ITT_goal,ITT_Delay,ITT_Delay*1.2);

	OilPressure_psi = (N1/100.0*0.25+(0.1-(OilTemp_degK-273.15)*0.1/80.0)*N1/100.0) / 7692.0e-6; //from MPa to psi
		OilTemp_degK = Seek(&OilTemp_degK, 353.15, 0.4-N1*0.001, 0.04);

	}
    else {
      phase = tpRun;
      Running = true;
      Starter = false;
      Cranking = false;
      FuelFlow_pph = 0;
	EngPower_HP=0.0;
      } 
    }
  else {                 // no start if N2 < 15% or Starved
    phase = tpOff;
    Starter = false;
    }

	ConsumeFuel();

  return EngPower_HP; 
}


//%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

double FGTurboProp::CalcFuelNeed(void)
{
  return FuelFlow_pph /3600 * State->Getdt() * Propulsion->GetRate();
}


//%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

double FGTurboProp::Seek(double *var, double target, double accel, double decel) {
  double v = *var;
  if (v > target) {
    v -= dt * decel;
    if (v < target) v = target;
  } else if (v < target) {
    v += dt * accel;
    if (v > target) v = target;
  }
  return v;
}

//%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

double FGTurboProp::ExpSeek(double *var, double target, double accel_tau, double decel_tau) {
// exponencial delay instead linear used in Seek
  double v = *var;
  if (v > target) {
		v = (v - target) * exp ( -dt / decel_tau) + target;
  } else if (v < target) {
		v = (target - v) * (1 - exp ( -dt / accel_tau)) + v;
  }
  return v;
}

//%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

void FGTurboProp::SetDefaults(void)
{
  Name = "Not defined";
  N1 = N2 = 0.0;
  Type = etTurboProp;
  MilThrust = 10000.0;
  MaxThrust = 10000.0;
  TSFC = 0.8;
  ATSFC = 1.7;
  IdleN1 = 30.0;
  IdleN2 = 60.0;
  MaxN1 = 100.0;
  MaxN2 = 100.0;
  Augmented = 0;
  AugMethod = 0;
  Injected = 0;
  BleedDemand = 0.0;
  ThrottleCmd = 0.0;
  InletPosition = 1.0;
  NozzlePosition = 1.0;
  Augmentation = false;
  Injection = false;
  Reversed = false;
  Cutoff = true;
  phase = tpOff;
  Stalled = false;
  Seized = false;
  Overtemp = false;
  Fire = false;
	Eng_ITT_degC = 0.0;

	GeneratorPower=true;
	Condition = 0;
	Ielu_intervent=false;

  Idle_Max_Delay = 1.0;
  
}

//%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

bool FGTurboProp::Load(FGConfigFile *Eng_cfg)
{
  string token;
	int rows, cols;

  IdleFF=-1;
  MaxStartingTime = 999999; //very big timeout -> infinite
  Ielu_max_torque=-1;

  Name = Eng_cfg->GetValue("NAME");
  Eng_cfg->GetNextConfigLine();
  int counter=0;

  while (Eng_cfg->GetValue() != string("/FG_TURBOPROP")) {
    *Eng_cfg >> token;

	  cout <<"Token:"<<token<<"\n";
	  
    if (token[0] == '<') token.erase(0,1); // Tables are read "<TABLE"

    if      (token == "MILTHRUST") *Eng_cfg >> MilThrust;
    else if (token == "MAXTHRUST") *Eng_cfg >> MaxThrust;
    else if (token == "TSFC") *Eng_cfg >> TSFC;
    else if (token == "ATSFC") *Eng_cfg >> ATSFC;
    else if (token == "IDLEN1") *Eng_cfg >> IdleN1;
    else if (token == "IDLEN2") *Eng_cfg >> IdleN2;
    else if (token == "MAXN1") *Eng_cfg >> MaxN1;
    else if (token == "MAXN2") *Eng_cfg >> MaxN2;
    else if (token == "BETARANGEEND") { *Eng_cfg >> BetaRangeThrottleEnd; BetaRangeThrottleEnd/=100.0; }
    else if (token == "REVERSEMAXPOWER") { *Eng_cfg >> ReverseMaxPower; ReverseMaxPower/=100.0; }
    else if (token == "AUGMENTED") *Eng_cfg >> Augmented;
    else if (token == "AUGMETHOD") *Eng_cfg >> AugMethod;
    else if (token == "INJECTED") *Eng_cfg >> Injected;
    else if (token == "MINTHROTTLE") *Eng_cfg >> MinThrottle;

    else if (token == "MAXPOWER") *Eng_cfg >> MaxPower;
    else if (token == "IDLEFUELFLOW") *Eng_cfg >> IdleFF;
    else if (token == "FUELCONSUMPTION") *Eng_cfg >> FuelConsumption;
    else if (token == "N1IDLE_MAX_DELAY") *Eng_cfg >> Idle_Max_Delay;

    else if (token == "MAXSTARTINGTIME") *Eng_cfg >> MaxStartingTime;
    else if (token == "STARTERN1") *Eng_cfg >> StarterN1;

    else if (token == "IELUMAXTORQUE") *Eng_cfg >> Ielu_max_torque;

    else if (token == "ITT_DELAY") *Eng_cfg >> ITT_Delay;

    else if (token == "ENG_POWER_RPM_N1") {
			*Eng_cfg >> rows >> cols;
			EngPower_RPM_N1 = new FGTable(rows, cols);
			*EngPower_RPM_N1 << *Eng_cfg;
		}
    else if (token == "ITT_N1") {
			*Eng_cfg >> rows >> cols;
			ITT_N1 = new FGTable(rows, cols);
			*ITT_N1 << *Eng_cfg;
		}


    else if (token == "TABLE") {
      if (counter++ == 0) Debug(0); // print engine specs prior to table read
      ThrustTables.push_back( new FGCoefficient(FDMExec) );
      ThrustTables.back()->Load(Eng_cfg);
    }
    else cerr << "Unhandled token in Engine config file: " << token << endl;
  }

  // Pre-calculations and initializations

  delay=1;
	N1_factor = MaxN1 - IdleN1;
  N2_factor = MaxN2 - IdleN2;
  OilTemp_degK = (Auxiliary->GetTotalTemperature() - 491.69) * 0.5555556 + 273.0;
  if (IdleFF==-1) IdleFF = pow(MilThrust, 0.2) * 107.0;  // just an estimate

	cout << "ENG POWER:" << EngPower_RPM_N1->GetValue(1200,90) << "\n";
	
  return true;
}

//%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

string FGTurboProp::GetEngineLabels(string delimeter)
{
  std::ostringstream buf;

  buf << Name << "_N1[" << EngineNumber << "]" << delimeter
      << Name << "_N2[" << EngineNumber << "]" << delimeter
      << Name << "__PwrAvailJVK[" << EngineNumber << "]" << delimeter
      << Thruster->GetThrusterLabels(EngineNumber, delimeter);

  return buf.str();
}

//%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

string FGTurboProp::GetEngineValues(string delimeter)
{
  std::ostringstream buf;

  buf << N1 << delimeter
      << N2 << delimeter
      << Thruster->GetThrusterValues(EngineNumber,delimeter);

  return buf.str();
}

//%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

void FGTurboProp::bindmodel()
{
  char property_name[80];

  snprintf(property_name, 80, "propulsion/n1[%u]", EngineNumber);
  PropertyManager->Tie( property_name, &N1);
  snprintf(property_name, 80, "propulsion/n2[%u]", EngineNumber);
  PropertyManager->Tie( property_name, &N2);

  snprintf(property_name, 80, "/controls/engines/engine[%u]/reverser_nase", EngineNumber);
  PropertyManager->Tie( property_name, &Reversed);

}

//%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

void FGTurboProp::unbind()
{
  char property_name[80];

  snprintf(property_name, 80, "propulsion/n1[%u]", EngineNumber);
  PropertyManager->Untie(property_name);
  snprintf(property_name, 80, "propulsion/n2[%u]", EngineNumber);
  PropertyManager->Untie(property_name);
}


//%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
//    The bitmasked value choices are as follows:
//    unset: In this case (the default) JSBSim would only print
//       out the normally expected messages, essentially echoing
//       the config files as they are read. If the environment
//       variable is not set, debug_lvl is set to 1 internally
//    0: This requests JSBSim not to output any messages
//       whatsoever.
//    1: This value explicity requests the normal JSBSim
//       startup messages
//    2: This value asks for a message to be printed out when
//       a class is instantiated
//    4: When this value is set, a message is displayed when a
//       FGModel object executes its Run() method
//    8: When this value is set, various runtime state variables
//       are printed out periodically
//    16: When set various parameters are sanity checked and
//       a message is printed out when they go out of bounds

void FGTurboProp::Debug(int from)
{
  if (debug_lvl <= 0) return;

  if (debug_lvl & 1) { // Standard console startup message output
    if (from == 0) { // Constructor

    }
    if (from == 2) { // called from Load()
	  cout << "\n ****MUJ MOTOR TURBOPROP****\n";
      cout << "\n    Engine Name: "         << Name << endl;
      cout << "      MilThrust:   "         << MilThrust << endl;
      cout << "      MaxThrust:   "         << MaxThrust << endl;
      cout << "      TSFC:        "         << TSFC << endl;
      cout << "      ATSFC:       "         << ATSFC << endl;
      cout << "      IdleN1:      "         << IdleN1 << endl;
      cout << "      IdleN2:      "         << IdleN2 << endl;
      cout << "      MaxN1:       "         << MaxN1 << endl;
      cout << "      MaxN2:       "         << MaxN2 << endl;
      cout << "      Augmented:   "         << Augmented << endl;
      cout << "      AugMethod:   "         << AugMethod << endl;
      cout << "      Injected:    "         << Injected << endl;
      cout << "      MinThrottle: "         << MinThrottle << endl;

      cout << endl;
    }
  }
  if (debug_lvl & 2 ) { // Instantiation/Destruction notification
    if (from == 0) cout << "Instantiated: FGTurboProp" << endl;
    if (from == 1) cout << "Destroyed:    FGTurboProp" << endl;
  }
  if (debug_lvl & 4 ) { // Run() method entry print for FGModel-derived objects
  }
  if (debug_lvl & 8 ) { // Runtime state variables
  }
  if (debug_lvl & 16) { // Sanity checking
  }
  if (debug_lvl & 64) {
    if (from == 0) { // Constructor
      cout << IdSrc << endl;
      cout << IdHdr << endl;
    }
  }
}
}
