// heading_indicator.cxx - a vacuum-powered heading indicator.
// Written by David Megginson, started 2002.
//
// This file is in the Public Domain and comes with no warranty.

//JVK
//There was only one kind of power source for gyro possible - vacuum system.
//Now it is possible to use vacuum or electrical power source depending on
//presence of <electrical-input> tag in "instrumentation.xml file".
//for ex. <electrical-input>/systems/electrical/outputs/head_ind[0]</electrical-input>

#include <simgear/compiler.h>
#include STL_IOSTREAM
#include STL_STRING
#include <sstream>

#include "heading_indicator.hxx"
#include <Main/fg_props.hxx>
#include <Main/util.hxx>


HeadingIndicator::HeadingIndicator ( SGPropertyNode *node )
    :
    name("heading-indicator"),
    num(0),
    vacuum_system("/systems/vacuum")
{
//JVK-added-line
    electrical = false;
    int i;
    for ( i = 0; i < node->nChildren(); ++i ) {
        SGPropertyNode *child = node->getChild(i);
        string cname = child->getName();
        string cval = child->getStringValue();
        if ( cname == "name" ) {
            name = cval;
        } else if ( cname == "number" ) {
            num = child->getIntValue();
        } else if ( cname == "vacuum-system" ) {
            vacuum_system = cval;
//JVK-added-lines
//set gyro type to electrical when <electrical-input> tag found.
        } else if ( cname == "electrical-input" ) {
            electrical_input = cval;
            electrical = true;
//JVK-end
        } else {
            SG_LOG( SG_INSTR, SG_WARN, "Error in heading-indicator config logic" );
            if ( name.length() ) {
                SG_LOG( SG_INSTR, SG_WARN, "Section = " << name );
            }
        }
    }
}

HeadingIndicator::HeadingIndicator ()
{
}

HeadingIndicator::~HeadingIndicator ()
{
}

void
HeadingIndicator::init ()
{
    string branch;
    branch = "/instrumentation/" + name;
    vacuum_system += "/suction-inhg";

    SGPropertyNode *node = fgGetNode(branch.c_str(), num, true );
    _offset_node = node->getChild("offset-deg", 0, true);
    _heading_in_node = fgGetNode("/orientation/heading-deg", true);
    _suction_node = fgGetNode(vacuum_system.c_str(), true);
    _heading_out_node = node->getChild("indicated-heading-deg", 0, true);
    _last_heading_deg = (_heading_in_node->getDoubleValue() +
                         _offset_node->getDoubleValue());
//JVK-added lines
    if (electrical)
        _electrical_power_node = fgGetNode(electrical_input.c_str(), true);
//JVK-end
}

void
HeadingIndicator::bind ()
{
    std::ostringstream temp;
    string branch;
    temp << num;
    branch = "/instrumentation/" + name + "[" + temp.str() + "]";

    fgTie((branch + "/serviceable").c_str(),
          &_gyro, &Gyro::is_serviceable, &Gyro::set_serviceable);
    fgTie((branch + "/spin").c_str(),
          &_gyro, &Gyro::get_spin_norm, &Gyro::set_spin_norm);
}

void
HeadingIndicator::unbind ()
{
    std::ostringstream temp;
    string branch;
    temp << num;
    branch = "/instrumentation/" + name + "[" + temp.str() + "]";

    fgUntie((branch + "/serviceable").c_str());
    fgUntie((branch + "/spin").c_str());
}

void
HeadingIndicator::update (double dt)
{
                                // Get the spin from the gyro
//JVK-added lines
//When gyro is electrical get the power from electrical node instead vacuum.
    if (electrical) {
		_gyro.set_power_norm(_electrical_power_node->getDoubleValue()>1?1:0);
	} else
//JVK-end
    _gyro.set_power_norm(_suction_node->getDoubleValue()/5.0);
    _gyro.update(dt);
    double spin = _gyro.get_spin_norm();

                                // Next, calculate time-based precession
    double offset = _offset_node->getDoubleValue();
    offset -= dt * (0.25 / 60.0); // 360deg/day
    while (offset < -360)
        offset += 360;
    while (offset > 360)
        offset -= 360;
    _offset_node->setDoubleValue(offset);

                                // TODO: movement-induced error

                                // Next, calculate the indicated heading,
                                // introducing errors.
    double factor = 0.01 / (spin * spin * spin * spin * spin * spin);
    double heading = _heading_in_node->getDoubleValue();

                                // Now, we have to get the current
                                // heading and the last heading into
                                // the same range.
    while ((heading - _last_heading_deg) > 180)
        _last_heading_deg += 360;
    while ((heading - _last_heading_deg) < -180)
        _last_heading_deg -= 360;

    heading = fgGetLowPass(_last_heading_deg, heading, dt/factor);
    _last_heading_deg = heading;

    heading += offset;
    while (heading < 0)
        heading += 360;
    while (heading > 360)
        heading -= 360;

    _heading_out_node->setDoubleValue(heading);
}

// end of heading_indicator.cxx
